using Unity.Lifetime;
using Unity;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;
using ProjectItCompany.Repositories;
using ProjectItCompany.Repositories.Implementations;
using Unity.Microsoft.Logging;
using Serilog;

namespace ProjectItCompany
{
    internal static class Program
    {
        /// <summary>
        ///  The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            // To customize application configuration such as set high DPI settings or default font,
            // see https://aka.ms/applicationconfiguration.
            ApplicationConfiguration.Initialize();
            Application.Run(CreateContainer().Resolve<FormItCompany>());
        }

        private static IUnityContainer CreateContainer()
        {
            var container = new UnityContainer();

            container.AddExtension(new LoggingExtension(CreateLoggerFactory()));

            container.RegisterType<ICustomerRepository, CustomerRepository>(new TransientLifetimeManager());
            container.RegisterType<IExecutorRepository, ExecutorRepository>(new TransientLifetimeManager());
            container.RegisterType<IProjectRepository, ProjectRepository>(new TransientLifetimeManager());
            container.RegisterType<IWageRepository, WageRepository>(new TransientLifetimeManager());
            container.RegisterType<IContractRepository, ContractRepository>(new TransientLifetimeManager());
            container.RegisterType<IConnectionString, ConnectionString>();
            return container;
        }

        private static LoggerFactory CreateLoggerFactory()
        {
            var loggerFactory = new LoggerFactory();
            loggerFactory.AddSerilog(new LoggerConfiguration()
                .ReadFrom.Configuration(new ConfigurationBuilder()
                .AddJsonFile("appsettings.json")
                .Build())
                .CreateLogger());
            return loggerFactory;
        }
    }
}