﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectItCompany.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectItCompany.Repositories.Implementations;

public class ProjectRepository : IProjectRepository
{
    private readonly IConnectionString _connectionString;
    private readonly ILogger<ProjectRepository> _logger;

    public ProjectRepository(IConnectionString connectionString, ILogger<ProjectRepository> logger)
    {
        _connectionString = connectionString;
        _logger = logger;
    }
    public void CreateProject(Project project)
    {
        _logger.LogInformation("Добавление объекта");
        _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(project));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryInsert = @"INSERT INTO project (NameProject, CustomerId, CompletedStages) 
                                VALUES (@NameProject, @CustomerId, @CompletedStages)";
            connection.Execute(queryInsert, project);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при добавлении объекта");
            throw;
        }
    }

    public void UpdateProject(Project project)
    {
        _logger.LogInformation("Редактирование объекта");
        _logger.LogDebug("Объект: {json}",
        JsonConvert.SerializeObject(project));
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryUpdate = @"UPDATE project SET
                                    NameProject=@NameProject,
                                    CustomerId=@CustomerId,
                                    CompletedStages=@CompletedStages
                                    WHERE ID=@id";
            connection.Execute(queryUpdate, project);
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при редактировании объекта");
            throw;
        }
    }

    public void DeleteProject(int id)
    {
        _logger.LogInformation("Удаление объекта");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var queryDelete = @"DELETE FROM project WHERE ID=@id";
            connection.Execute(queryDelete, new { id });
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при удалении объекта");
            throw;
        }
    }

    public Project ReadProjectById(int id)
    {
        _logger.LogInformation("Получение объекта по идентификатору");
        _logger.LogDebug("Объект: {id}", id);
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = @"SELECT * FROM project WHERE ID=@id";
            var project = connection.QueryFirst<Project>(querySelect, new
            {
                id
            });
            _logger.LogDebug("Найденный объект: {json}",
            JsonConvert.SerializeObject(project));
            return project;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при поиске объекта");
            throw;
        }
    }

    public IEnumerable<Project> ReadProjects()
    {
        _logger.LogInformation("Получение всех объектов");
        try
        {
            using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
            var querySelect = "SELECT * FROM project";
            var projects = connection.Query<Project>(querySelect);
            _logger.LogDebug("Полученные объекты: {json}",
            JsonConvert.SerializeObject(projects));
            return projects;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при чтении объектов");
            throw;
        }
    }
}
